@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import keyword
+import logging
 import operator
 import sys
 import warnings
@@ -12,7 +13,12 @@ from typing import TYPE_CHECKING, Any, TypeVar
 import awkward as ak
 import dask.config
 import numpy as np
-from awkward._typetracer import MaybeNone, OneOf, UnknownScalar
+from awkward._nplikes.typetracer import (
+    MaybeNone,
+    OneOf,
+    TypeTracerArray,
+    is_unknown_scalar,
+)
 from awkward.highlevel import _dir_pattern
 from dask.base import DaskMethodsMixin, dont_optimize, is_dask_collection, tokenize
 from dask.blockwise import BlockwiseDep
@@ -48,8 +54,10 @@ if TYPE_CHECKING:
 T = TypeVar("T")
 
 
-def _finalize_array(results: Sequence[Any]) -> Any:
+log = logging.getLogger(__name__)
 
+
+def _finalize_array(results: Sequence[Any]) -> Any:
     # special cases for length 1 results
     if len(results) == 1:
         if isinstance(results[0], (int, ak.Array)):
@@ -157,9 +165,9 @@ class Scalar(DaskMethodsMixin):
         return (self._name, 0)
 
     def _check_meta(self, m: Any) -> Any | None:
-        if not isinstance(m, (MaybeNone, UnknownScalar, OneOf)):
-            raise TypeError(f"meta must be a typetracer object, not a {type(m)}")
-        return m
+        if isinstance(m, (MaybeNone, OneOf)) or is_unknown_scalar(m):
+            return m
+        raise TypeError(f"meta must be a typetracer object, not a {type(m)}")
 
     @property
     def dtype(self) -> np.dtype | None:
@@ -260,7 +268,7 @@ def new_scalar_object(dsk: HighLevelGraph, name: str, *, meta: Any) -> Scalar:
 
     """
     if meta is None:
-        meta = UnknownScalar(np.dtype(None))
+        meta = TypeTracerArray._new(dtype=np.dtype(None), shape=())
     return Scalar(dsk, name, meta, known_value=None)
 
 
@@ -310,7 +318,9 @@ def new_known_scalar(
         dtype = np.dtype(dtype)
     llg = {(name, 0): s}
     hlg = HighLevelGraph.from_collections(name, llg, dependencies=())
-    return Scalar(hlg, name, meta=UnknownScalar(dtype), known_value=s)
+    return Scalar(
+        hlg, name, meta=TypeTracerArray._new(dtype=dtype, shape=()), known_value=s
+    )
 
 
 class Record(Scalar):
@@ -699,15 +709,18 @@ class Array(DaskMethodsMixin, NDArrayOperatorsMixin):
             else:
                 m = to_meta([where])[0]
                 meta = self._meta[m]
-        return self.map_partitions(
+        return map_partitions(
             operator.getitem,
+            self,
             where,
             meta=meta,
             output_divisions=1,
             label=label,
         )
 
-    def _getitem_outer_boolean_lazy_array(self, where: Array | tuple[Any, ...]) -> Any:
+    def _getitem_outer_bool_or_int_lazy_array(
+        self, where: Array | tuple[Any, ...]
+    ) -> Any:
         ba = where if isinstance(where, Array) else where[0]
         if not compatible_partitions(self, ba):
             raise IncompatiblePartitions("getitem", self, ba)
@@ -716,7 +729,7 @@ class Array(DaskMethodsMixin, NDArrayOperatorsMixin):
         if self._meta is not None:
             if isinstance(where, tuple):
                 raise DaskAwkwardNotImplemented(
-                    "tuple style input boolean selection is not supported."
+                    "tuple style input boolean/int selection is not supported."
                 )
             elif isinstance(where, Array):
                 new_meta = self._meta[where._meta]
@@ -784,7 +797,7 @@ class Array(DaskMethodsMixin, NDArrayOperatorsMixin):
             return result
 
         # otherwise make sure we have one of the other potential results.
-        if not isinstance(new_meta, (ak.Record, UnknownScalar, OneOf, MaybeNone)):
+        if not isinstance(new_meta, (ak.Record, TypeTracerArray, OneOf, MaybeNone)):
             raise DaskAwkwardNotImplemented("Key type not supported for this array.")
 
         token = tokenize(partition, where)
@@ -825,15 +838,14 @@ class Array(DaskMethodsMixin, NDArrayOperatorsMixin):
                 dtype = where[0].layout.dtype.type
             except AttributeError:
                 dtype = where[0].layout.content.dtype.type
-            if issubclass(dtype, (np.bool_, bool)):
-                return self._getitem_outer_boolean_lazy_array(where)
+            if issubclass(dtype, (np.bool_, bool, np.int64, np.int32, int)):
+                return self._getitem_outer_bool_or_int_lazy_array(where)
 
         raise DaskAwkwardNotImplemented(
             f"Array.__getitem__ doesn't support multi object: {where}"
         )
 
     def _getitem_single(self, where: Any) -> Array:
-
         # a single string
         if isinstance(where, str):
             return self._getitem_outer_str_or_list(where, label=where)
@@ -850,8 +862,8 @@ class Array(DaskMethodsMixin, NDArrayOperatorsMixin):
                 dtype = where.layout.dtype.type
             except AttributeError:
                 dtype = where.layout.content.dtype.type
-            if issubclass(dtype, (np.bool_, bool)):
-                return self._getitem_outer_boolean_lazy_array(where)
+            if issubclass(dtype, (np.bool_, bool, np.int64, np.int32, int)):
+                return self._getitem_outer_bool_or_int_lazy_array(where)
 
         # an empty slice
         elif is_empty_slice(where):
@@ -1051,10 +1063,10 @@ class Array(DaskMethodsMixin, NDArrayOperatorsMixin):
 
         return to_delayed(self, optimize_graph=optimize_graph)
 
-    def to_dask_array(self) -> DaskArray:
+    def to_dask_array(self, optimize_graph: bool = True) -> DaskArray:
         from dask_awkward.lib.io.io import to_dask_array
 
-        return to_dask_array(self)
+        return to_dask_array(self, optimize_graph=optimize_graph)
 
     def to_parquet(
         self,
@@ -1073,10 +1085,14 @@ class Array(DaskMethodsMixin, NDArrayOperatorsMixin):
 
 
 def compute_typetracer(dsk: HighLevelGraph, name: str) -> ak.Array:
-    if dask.config.get("awkward.compute-unknown-meta"):
-        key = (name, 0)
-        return typetracer_array(Delayed(key, dsk.cull({key}), layer=name).compute())
-    return empty_typetracer()
+    key = (name, 0)
+    return typetracer_array(
+        Delayed(
+            key,
+            dsk.cull({key}),
+            layer=name,
+        ).compute()
+    )
 
 
 def new_array_object(
@@ -1128,7 +1144,10 @@ def new_array_object(
         divs = divisions
 
     if meta is None:
-        actual_meta = compute_typetracer(dsk, name)
+        if dask.config.get("awkward.compute-unknown-meta"):
+            actual_meta = compute_typetracer(dsk, name)
+        else:
+            actual_meta = empty_typetracer()
     else:
         if not isinstance(meta, ak.Array):
             raise TypeError("meta must be an instance of an Awkward Array.")
@@ -1277,17 +1296,7 @@ def map_partitions(
     ]
 
     if meta is None:
-        metas = to_meta(args)
-        try:
-            meta = fn(*metas, **kwargs)
-        except Exception:
-            if dask.config.get("awkward.compute-unknown-meta"):
-                warnings.warn(
-                    "metadata could not be determined; "
-                    "a compute on the first partition will occur.",
-                    UserWarning,
-                )
-            pass
+        meta = map_meta(fn, *args, **kwargs)
 
     hlg = HighLevelGraph.from_collections(
         name,
@@ -1478,7 +1487,7 @@ def is_typetracer(obj: Any) -> bool:
     Typetracers can be one of these categories:
     - Array
     - Record
-    - UnknownScalar
+    - TypeTracerArray
     - MaybeNone
     - OneOf
 
@@ -1497,10 +1506,10 @@ def is_typetracer(obj: Any) -> bool:
     if isinstance(obj, (ak.Array, ak.Record)):
         backend = obj.layout.backend
 
-        if not backend.nplike.known_shape and not backend.nplike.known_data:
+        if not backend.nplike.known_data:
             return True
     # scalar-like typetracer
-    elif isinstance(obj, (UnknownScalar, MaybeNone, OneOf)):
+    elif is_unknown_scalar(obj) or isinstance(obj, (MaybeNone, OneOf)):
         return True
     return False
 
@@ -1559,6 +1568,63 @@ def to_meta(objects: Sequence[Any]) -> tuple[Any, ...]:
     return tuple(map(meta_or_identity, objects))
 
 
+def length_zero_array_or_identity(obj: Any) -> Any:
+    if is_awkward_collection(obj):
+        return obj._meta.layout.form.length_zero_array()
+    return obj
+
+
+def to_length_zero_arrays(objects: Sequence[Any]) -> tuple[Any, ...]:
+    return tuple(map(length_zero_array_or_identity, objects))
+
+
+def map_meta(fn: Callable, *args: Any, **kwargs: Any) -> ak.Array | None:
+    metas = to_meta(args)
+    try:
+        meta = fn(*metas, **kwargs)
+        return meta
+    except Exception as err:
+        # if compute-unknown-meta is False then we don't care about
+        # this failure and we return None.
+        if not dask.config.get("awkward.compute-unknown-meta"):
+            return None
+
+        # if the metadata function call failed and raise-failed-meta
+        # is True, then we want to raise the exception here.
+        if dask.config.get("awkward.raise-failed-meta"):
+            log.debug("metadata determination failed: %s" % err)
+            raise
+
+        # if the metadata function failed and we want to move on to
+        # trying the length zero array calculation then we log a
+        # warning and pass to the next try-except block.
+        else:
+            log.warning(
+                "function call on just metas failed; will try length zero array technique"
+            )
+        pass
+    try:
+        lzas = to_length_zero_arrays(args)
+        meta = typetracer_from_form(fn(*lzas, **kwargs).layout.form)
+        return meta
+    except Exception:
+        # if compute-unknown-meta is True and we've gotten to this
+        # point, we want to throw a warning because a compute is going
+        # to happen as a consequence of us not being able to determine
+        # metadata.
+        if dask.config.get("awkward.compute-unknown-meta"):
+            extras = (
+                f"function call: {fn}\n" f"metadata: {metas}\n" f"kwargs: {kwargs}\n"
+            )
+            warnings.warn(
+                "metadata could not be determined; "
+                "a compute on the first partition will occur.\n"
+                f"{extras}",
+                UserWarning,
+            )
+    return None
+
+
 def typetracer_array(a: ak.Array | Array) -> ak.Array:
     """Retrieve the typetracer Array from a concrete or lazy instance.
 
@@ -1590,7 +1656,6 @@ def compatible_partitions(*args: Array) -> bool:
 
     In operations where the blocks of multiple collections are used
     simultaneously, we need the collections to be equally partitioned.
-
     If the first argument has known divisions, other collections with
     known divisions will be tested against the first arguments
     divisions.
